@@ -1,4 +1,4 @@
-# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2003-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of logilab-database.
@@ -23,6 +23,7 @@ Currently support:
 - postgresql (pgdb, psycopg, psycopg2, pyPgSQL)
 - mysql (MySQLdb)
 - sqlite (pysqlite2, sqlite, sqlite3)
+- sqlserver 2000, 2005, 2008 (pyodbc, adodbapi)
 
 just use the `get_connection` function from this module to get a
 wrapped connection.  If multiple drivers for a database are available,
@@ -33,10 +34,12 @@ Additional helpers are also provided for advanced functionalities such
 as listing existing users or databases, creating database... Get the
 helper for your database using the `get_db_helper` function.
 """
+from __future__ import with_statement
 
 __docformat__ = "restructuredtext en"
 
 import sys
+import threading
 import logging
 from datetime import datetime, time
 
@@ -47,12 +50,16 @@ _LOGGER = logging.getLogger('logilab.database')
 
 USE_MX_DATETIME = False
 
+_LOAD_MODULES_LOCK = threading.Lock()
+
 _PREFERED_DRIVERS = {}
 _ADV_FUNC_HELPER_DIRECTORY = {}
 
 def _ensure_module_loaded(driver):
     if driver in ('postgres', 'sqlite', 'mysql', 'sqlserver2005'):
-        __import__('logilab.database.%s' % driver)
+        with _LOAD_MODULES_LOCK:
+            __import__('logilab.database.%s' % driver)
+
 # main functions ###############################################################
 
 def get_db_helper(driver):
@@ -83,7 +90,7 @@ def get_connection(driver='postgres', host='', database='', user='',
         adapter = _ADAPTER_DIRECTORY.get_adapter(driver, modname)
     except NoAdapterFound, err:
         if not quiet:
-            msg = 'No Adapter found for %s, using default one' 
+            msg = 'No Adapter found for %s, using default one'
             _LOGGER.warning(msg, err.objname)
         adapted_module = DBAPIAdapter(module, pywrap)
     else:
@@ -106,16 +113,17 @@ def set_prefered_driver(driver, module, _drivers=_PREFERED_DRIVERS):
     _drivers is a optional dictionary of drivers
     """
     _ensure_module_loaded(driver)
-    try:
-        modules = _drivers[driver]
-    except KeyError:
-        raise UnknownDriver('Unknown driver %s' % driver)
-    # Remove module from modules list, and re-insert it in first position
-    try:
-        modules.remove(module)
-    except ValueError:
-        raise UnknownDriver('Unknown module %s for %s' % (module, driver))
-    modules.insert(0, module)
+    with _LOAD_MODULES_LOCK:
+        try:
+            modules = _drivers[driver]
+        except KeyError:
+            raise UnknownDriver('Unknown driver %s' % driver)
+        # Remove module from modules list, and re-insert it in first position
+        try:
+            modules.remove(module)
+        except ValueError:
+            raise UnknownDriver('Unknown module %s for %s' % (module, driver))
+        modules.insert(0, module)
 
 
 # unified db api ###############################################################
@@ -175,18 +183,19 @@ def _import_driver_module(driver, drivers, quiet=True):
     """
     if not driver in drivers:
         raise UnknownDriver(driver)
-    for modname in drivers[driver]:
-        try:
-            if not quiet:
-                _LOGGER.info('Trying %s', modname)
-            module = load_module_from_name(modname, use_sys=False)
-            break
-        except ImportError:
-            if not quiet:
-                _LOGGER.warning('%s is not available', modname)
-            continue
-    else:
-        raise ImportError('Unable to import a %s module' % driver)
+    with _LOAD_MODULES_LOCK:
+        for modname in drivers[driver]:
+            try:
+                if not quiet:
+                    _LOGGER.info('Trying %s', modname)
+                module = load_module_from_name(modname, use_sys=False)
+                break
+            except ImportError:
+                if not quiet:
+                    _LOGGER.warning('%s is not available', modname)
+                continue
+        else:
+            raise ImportError('Unable to import a %s module' % driver)
     return module, modname
 
 
@@ -276,6 +285,8 @@ class DBAPIAdapter(object):
     row_is_mutable = False
     # True if the fetch*() methods return unicode and not binary strings
     returns_unicode = False
+    # True is the backend support COPY FROM method
+    support_copy_from = False
 
     def __init__(self, native_module, pywrap=False):
         """
@@ -324,6 +335,8 @@ class DBAPIAdapter(object):
             transform = bool
         elif typecode == self.BINARY and binarywrap is not None:
             transform = binarywrap
+        elif typecode == self.DATETIME: # XXX only for TZDatetime w/ backend returning some tzinfo
+            transform = lambda v: utcdatetime(v) if isinstance(v, datetime) else v
         elif typecode == self.UNKNOWN:
             # may occurs on constant selection for instance (e.g. SELECT 'hop')
             # with postgresql at least
@@ -367,7 +380,7 @@ class DBAPIAdapter(object):
         for i, coldescr in enumerate(description):
             transform = self._transformation_callback(coldescr, encoding, binarywrap)
             if transform is not None:
-                transformations.append((i, transform))
+                transformations.append( (i, transform) )
         return transformations
 
     def process_value(self, value, description, encoding='utf-8',
@@ -531,6 +544,12 @@ class MINUTE(ExtractDateField):
 class SECOND(ExtractDateField):
     field = 'SECOND'
 
+class EPOCH(ExtractDateField):
+    """Return EPOCH timestamp from a datetime/date ;
+    return number of seconds for an interval.
+    """
+    field = 'EPOCH'
+
 class WEEKDAY(FunctionDescr):
     """Return the day of the week represented by the date.
 
@@ -622,7 +641,7 @@ for func_class in (
     ABS, RANDOM,
     UPPER, LOWER, SUBSTRING, LENGTH,
     DATE,
-    YEAR, MONTH, DAY, HOUR, MINUTE, SECOND, WEEKDAY, AT_TZ,
+    YEAR, MONTH, DAY, HOUR, MINUTE, SECOND, WEEKDAY, EPOCH, AT_TZ,
     # cast functions
     CAST,
     # keyword function
@@ -751,7 +770,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
 
     def backup_commands(self, backupfile, keepownership=True,
                         dbname=None, dbhost=None, dbport=None, dbuser=None):
-        """return a list of commands to backup the given database.
+        """Return a list of commands to backup the given database.
 
         Each command may be given as a list or as a string. In the latter case,
         expected to be used with a subshell (for instance using `os.system(cmd)`
@@ -762,7 +781,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
     def restore_commands(self, backupfile, keepownership=True, drop=True,
                          dbname=None, dbhost=None, dbport=None, dbuser=None,
                          dbencoding=None):
-        """return a list of commands to restore a backup of the given database
+        """Return a list of commands to restore a backup of the given database.
 
 
         Each command may be given as a list or as a string. In the latter case,
@@ -771,17 +790,36 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         """
         raise NotImplementedError('not supported by this DBMS')
 
-    # helpers to standardize SQL according to the database
+    # helpers to standardize SQL according to the database #####################
+
     def sql_current_date(self):
+        """Return sql for the current date.
+
+        Take care default implementation return date at the beginning of the
+        transaction on some backend (eg postgres)
+        """
         return 'CURRENT_DATE'
 
     def sql_current_time(self):
+        """Return sql for the current time.
+
+        Take care default implementation return time at the beginning of the
+        transaction on some backend (eg postgres)
+        """
         return 'CURRENT_TIME'
 
     def sql_current_timestamp(self):
+        """Return sql for the current date and time.
+
+        Take care default implementation return date and time at the beginning
+        of the transaction on some backend (eg postgres)
+        """
         return 'CURRENT_TIMESTAMP'
 
     def sql_concat_string(self, lhs, rhs):
+        """Return sql for concatenating given arguments (expected to be
+        evaluated as string when executing the query).
+        """
         return '%s || %s' % (lhs, rhs)
 
     def sql_regexp_match_expression(self, pattern):
