@@ -92,11 +92,18 @@ class _Psycopg2Adapter(_PsycopgAdapter):
     # "select typname from pg_type where oid=705";
     UNKNOWN = 705
     returns_unicode = True
+    # True is the backend support COPY FROM method
+    support_copy_from = True
 
     def __init__(self, native_module, pywrap=False):
         from psycopg2 import extensions
         extensions.register_type(extensions.UNICODE)
-        extensions.register_type(extensions.UNICODEARRAY)
+        try:
+            unicodearray = extensions.UNICODEARRAY
+        except AttributeError:
+            from psycopg2 import _psycopg
+            unicodearray = _psycopg.UNICODEARRAY
+        extensions.register_type(unicodearray)
         self.BOOLEAN = extensions.BOOLEAN
         db.DBAPIAdapter.__init__(self, native_module, pywrap)
         self._init_psycopg2()
@@ -127,6 +134,17 @@ class _Psycopg2Adapter(_PsycopgAdapter):
             #extensions.register_adapter(cStringIO.StringIO, adapt_stringio)
 
 
+class _Psycopg2CtypesAdapter(_Psycopg2Adapter):
+    """psycopg2-ctypes adapter
+
+    cf. https://github.com/mvantellingen/psycopg2-ctypes
+    """
+    def __init__(self, native_module, pywrap=False):
+        # install psycopg2 compatibility
+        from psycopg2ct import compat
+        compat.register()
+        _Psycopg2Adapter.__init__(self, native_module, pywrap)
+
 class _PgsqlAdapter(db.DBAPIAdapter):
     """Simple pyPgSQL Adapter to DBAPI
     """
@@ -150,12 +168,13 @@ class _PgsqlAdapter(db.DBAPIAdapter):
 
 db._PREFERED_DRIVERS['postgres'] = [
     #'logilab.database._pyodbcwrap',
-    'psycopg2', 'psycopg', 'pgdb', 'pyPgSQL.PgSQL',
+    'psycopg2', 'psycopg2ct', 'psycopg', 'pgdb', 'pyPgSQL.PgSQL',
     ]
 db._ADAPTER_DIRECTORY['postgres'] = {
     'pgdb' : _PgdbAdapter,
     'psycopg' : _PsycopgAdapter,
     'psycopg2' : _Psycopg2Adapter,
+    'psycopg2ct' : _Psycopg2CtypesAdapter,
     'pyPgSQL.PgSQL' : _PgsqlAdapter,
     }
 
@@ -219,6 +238,15 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         cmds.append(cmd)
         return cmds
 
+    def sql_current_date(self):
+        return 'CAST(clock_timestamp() AS DATE)'
+
+    def sql_current_time(self):
+        return 'CAST(clock_timestamp() AS TIME)'
+
+    def sql_current_timestamp(self):
+        return 'clock_timestamp()'
+
     def sql_regexp_match_expression(self, pattern):
         """pattern matching using regexp"""
         return "~ %s" % (pattern)
@@ -410,13 +438,14 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
 
         Require extensions to be already in.
         """
-        # XXX create GIN or GIST index, see FTS wiki
         return """
 CREATE table appears(
   uid     INTEGER PRIMARY KEY NOT NULL,
   words   tsvector,
   weight  FLOAT
 );
+
+CREATE INDEX appears_words_idx ON appears USING gin(words);
 """
 
     def sql_drop_fti(self):
